import attack
import config

class Node:
  def __init__(self, name, time):
    self.name = name
    self.time = time
    self.links = {}
  def add_link(self, move, node):
    self.links[move] = node

class Automaton:
  def __init__(self, character, datafile = "data/flowchart.txt"):
    self.character = character
    self.print_state_enabled = False
  
    self.nodes = {}
    self.node_groups = {}
    
    self.time_pass = 0.0
    
    current_animation = "default"
    reading_links = False
    reading_groups = False
    for line in open(datafile):
      line = line.strip()
      if line == "":
        if reading_groups:
          reading_links = True
          reading_groups = False
        else:
          reading_groups = True
        continue
      if line[0] == "#":
        # A comment, ignore it
        continue
      
      if reading_groups:
        fields = line.split()
        # Cut off the colon as well
        group_name = fields[0][:-1]
        group = []
        for name in fields[1:]:
          group.append(self.nodes[name])
        self.node_groups[group_name] = group
      elif reading_links:
        name1, move, name2 = line.split()
        node_list1 = self.get_node_list(name1)
        node_list2 = self.get_node_list(name2)        
        
        for node1 in node_list1:
          for node2 in node_list2:
            node1.add_link(move, node2)
      else:
        first, second = line.split()
        if first == "Animation":
          current_animation = second
        else:
          name, time = first, second
          node = Node(name, float(time))
          node.animation = current_animation[:]
          self.nodes[name] = node
        
    self.default_node = self.nodes["Default"]    
        
    self.node = self.default_node    
    self.time = 0.0
    
    self.print_state()
  
  def get_node_list(self, name):
    if name == "*":
      node_list = []
      for temp_name in self.nodes:
        node = self.nodes[temp_name]
        node_list.append(node)
      return node_list
    elif name in self.node_groups:
      return self.node_groups[name]
    else:
      return [self.nodes[name]]

  def print_state(self):
    if self.print_state_enabled:
      print "State: %s (%f) - Time: %f" % (self.node.name, self.node.time, self.time)
  
  def is_moving(self):
    return self.node in self.node_groups["Moving"]
  def is_walking(self):
    return self.node in self.node_groups["Walking"]
  def is_running(self):
    return self.node.name == "Run"
  def is_stunned(self):
    return self.node.name == "Stun"
  def is_safe(self):
    return self.node in self.node_groups["Safe"]
  def is_controllable(self):
    return self.node not in self.node_groups["Safe"]
  def is_trying_attack(self):
    return self.node in self.node_groups["Attack_process"]
  def is_attack(self):
    return self.node in self.node_groups["Attacks"]
  def is_falling(self):
    return self.node.name == "Falling"
  def is_on_ground(self):
    return not self.node in self.node_groups["InAir"]
  def is_knocked_down(self):
    return self.node.name == "KnockedDown"

  def get_attack_name(self):
    return self.node.name
  def get_animation(self):
    return self.node.animation
  
  def set_state(self, move):
    if move in self.node.links:
      self.node = self.node.links[move]
      self.time = 0.0
      self.print_state()
      return True
    else:
      return False
      
  def tick(self, time_delta):
    # Print out the state every second
    self.time_pass += time_delta
    if self.time_pass > 1:
      self.print_state()
      self.time_pass -= 1
      
    if self.node.time == 0.0:
      return
      
    if self.is_attack():
        if self.node.name in attack.attacks:
          self.character.client.apply_attack(attack.attacks[self.node.name], self.character)
      
    self.time += time_delta
    if self.time >= self.node.time:
      if not self.set_state("timeout"):
        self.node = self.default_node
        self.time = 0.0
        self.print_state()
    
