import sound_effects
import config
import images
import particle
import game_object
from rect import Rect
import math
from vector import Vector,Vector3
from automata import Automaton
import pyglet
import pyglet.sprite

FACING_LEFT, FACING_RIGHT = range(2)

class Character(game_object.Game_object):
  def __init__(self, pos, facing = FACING_RIGHT):
    super (Character,self).__init__(pos)
    self.current_animation = ""
    self.current_animation_facing = FACING_LEFT
    self.walk_acc = 250
    self.max_speed = 300

    self.vertical_speed = 50
    self.jump_velocity = 200
    self.sprite = None
    self.shadow = pyglet.sprite.Sprite (images.shadow)

    self.z_moved = False
    
    self.gravity = 300
    
    self.debug = False
    
    self.state = Automaton(self)
    self.facing = facing
    
    self.full_health = 1000
    self.health = self.full_health

    self.core_rect = Rect (-20.0,-14.0,20.0,55.0)
    self.size = Vector (128,128)
    
    self.last_enemy_hit = None
  
  def weight(self):
    return self.health / float(self.full_health)
    
  def set_max_speed(self, max_speed):
    self.max_speed = max_speed
  def tick(self, time_delta):
    self.state.tick(time_delta)
    self.z_moved = False 
    # x-movement
    if (self.state.is_walking()):
      acc_multiplier = 1
      max_speed_multiplier = 1
    elif (self.state.is_running()):
      acc_multiplier = 2
      max_speed_multiplier = 2
    else:
      acc_multiplier = 0
      max_speed_multiplier = 1
    
    if (self.facing == FACING_LEFT):
      acc_multiplier *= -1
    else:
      acc_multiplier *= 1
    
    velocity_delta = self.walk_acc * time_delta * acc_multiplier
    self.vel.x += velocity_delta

    max_speed = self.max_speed * max_speed_multiplier
    if self.state.is_moving() and self.state.is_on_ground():
      if (self.vel.x > max_speed):
        self.vel.x = max_speed
      if (self.vel.x < -max_speed):
        self.vel.x = -max_speed

    if self.debug:
      print "max_speed = ",max_speed

    if (not self.state.is_moving() and self.state.is_on_ground()):
        self.vel.x *= math.pow (0.003,time_delta)
    if self.debug:
      print velocity_delta

    # y-movement
    self.vel.y -= self.gravity * time_delta
    if (self.debug):
      print self.vel 
    
    # Move!
    self.pos += self.vel * time_delta
    if self.pos.y < 0:
      self.vel.y = 0
      self.pos.y = 0
      self.land()
    
    # Check if you've been knocked out
    if not self.dying:
      if self.pos.x < 0 or self.pos.x + self.size.x > self.client.world_size.x:
        if self.pos.y > self.client.fence_height:
          # Yes, you have, keep going and you're dead
          self.dying = True
        else:
          # Stop all x momentum
          self.vel.x = -self.vel.x
          if self.pos.x < 0:
            self.pos.x = 0
          else: # if self.pos.x + self.size.x > self.client.world_size.x:
            self.pos.x = self.client.world_size.x - self.size.x

  def move_z(self, distance):
    if self.state.is_controllable():
      self.z_moved = True
      self.pos.z += distance
      if self.pos.z < 0:
        self.pos.z = 0
      elif self.pos.z > self.client.world_size.z:
        self.pos.z = self.client.world_size.z
  def move_x(self, distance):
    if self.state.is_controllable():
      self.vel.x += distance

  def get_world_core_rect (self):
    return self.core_rect + self.get_centre().get_xy()

  def update_animation (self):
    new_animation = self.state.get_animation()
    #slight hack:
    if new_animation == "Standing" and self.z_moved:
      new_animation = "Walking"
    if new_animation != self.current_animation or self.facing != self.current_animation_facing:
      if self.facing == FACING_RIGHT:
        image = images.character_images["%s_%s"%(self.__class__.__name__,new_animation)]
      else:
        image = images.character_images["Reversed_%s_%s"%(self.__class__.__name__,new_animation)]
      if (self.sprite):
        self.sprite.image = image
      else:
        self.sprite = pyglet.sprite.Sprite(image)

      self.current_animation = new_animation
      self.current_animation_facing = self.facing

  def update_sprite (self, screen_pos):
    super (Character,self).update_sprite(screen_pos)
    self.shadow.x = self.pos.x - screen_pos.x + 43
    self.shadow.y = self.pos.z
    self.update_animation ()

  def draw2 (self, screen_pos):
    if (config.draw_attack_rects):
      pyglet.gl.glColor3f(1.0, 0, 0)
      (self.core_rect + self.get_viewport_centre_pos() - screen_pos).draw ()
      pyglet.gl.glColor3f(1.0, 1.0, 1.0)

  def face_left(self):
    return self.change_facing(FACING_LEFT)
  def face_right(self):
    return self.change_facing(FACING_RIGHT)
    
  def change_facing(self, facing):
    if self.state.is_attack():
      return False
    else:
      self.facing = facing
      return True
  

  def move_direction(self):
    return self.state.set_state("direction")

  def move_opposite(self):
    return self.state.set_state("opposite")
  
  def release(self):
    return self.state.set_state("release")
  
  def high_attack(self):
    return self.state.set_state("high")

  def low_attack(self):
    return self.state.set_state("low")
  
  def jump(self):
    return self.state.set_state("jump")
    
  def land(self):
    if self.dying:
      self.dead = True
    return self.state.set_state("land")
    
  def hit(self, damage, vel):
    if not self.state.is_safe():
      sound_effects.play_punch() 
      self.health -= damage
      if self.health < 1:
        self.health = 1
      self.client.add_number_particle(damage,self.get_centre() + Vector3(0,self.size.y/2,0))
      result = self.state.set_state("hit")
      if self.state.is_falling():
        min_weight = max(self.weight(), 0.1)
        self.vel += vel / min_weight
    return False

