import math


class Vector:
  def __init__(self, x, y):
    self.x = x
    self.y = y
  def clone(self):
    return Vector(self.x, self.y)
  def distance(self, other):
    return math.sqrt((self.x - other.x) * (self.x - other.x) + (self.y - other.y) * (self.y - other.y) )

  def is_null(self):
    return self.x == 0 and self.y == 0 

  def length(self):
    return math.sqrt(self.x * self.x + self.y * self.y)

  def scalar_add(self, scalar, scalar_y = None):
    if not scalar_y:
      return Vector(self.x + scalar,
        self.y + scalar)
    return Vector(self.x + scalar,
      self.y + scalar_y)

  def normalise(self):
    length = self.length()
    if length == 0:
      return
    self.x /= length
    self.y /= length

  def scale(self, x, y):
    return Vector(
      self.x / x,
      self.y / y,
    )
  def scalar_sub(self, scalar_x, scalar_y):
    return Vector(self.x - scalar_x,
      self.y - scalar.y
    )
  def rotate (self,degrees):
    '''anti clockwise'''
    rad = math.pi * degrees/180
    return Vector(
        math.cos(rad)*self.x - math.sin(rad)*self.y,
        math.sin(rad)*self.x + math.cos(rad)*self.y,
        )
    
  def inside_quad(self, center, width, height):
    return self.x > center.x - width/2 and self.x < center.x + width/2 and self.y > center.y - height/2 and self.y < center.y + height/2

  def __sub__(self, other):
    return Vector(
      self.x - other.x,
      self.y - other.y,
    )
    
  def __add__(self, other):
    return Vector(
      self.x + other.x,
      self.y + other.y,
    )
  def __iadd__(self, other):
    self.x += other.x
    self.y += other.y
    return self

  def __mul__(self, scalar):
    return Vector(
      self.x * scalar,
      self.y * scalar, 
    )
  
  def __div__(self, scalar):
    return Vector(
      self.x / scalar,
      self.y / scalar, 
    )
  
  def __eq__(self, other):
    return self.x == other.x and self.y == other.y 

  def angle_from_x_axis(self):
    if self.x == 0 and self.y == 0:
      return 0
    elif self.x > 0 and self.y < 0:
      return 360 - math.degrees(math.acos(self.x / self.length()))
    elif self.x <= 0 and self.y < 0:
      return 180 + math.degrees(math.acos(-self.x / self.length()))
    elif self.x <= 0 and self.y >= 0:
      return 180 - math.degrees(math.acos(-self.x / self.length()))
    else:
      return math.degrees(math.acos(self.x / self.length()))
      
  def __str__(self):
    return "Vector(%f, %f)" % (self.x, self.y)

  
class Vector3:
  def __init__(self, x = 0, y = 0, z = 0, vector_2d = None):
    self.x = x
    self.y = y
    self.z = z
    if vector_2d:
      self.x = vector_2d.x
      self.y = vector_2d.y
    
  def clone(self):
    return Vector3(self.x, self.y, self.z)
  def distance(self, other):
    return math.sqrt((self.x - other.x) * (self.x - other.x) + (self.y - other.y) * (self.y - other.y) +  (self.z - other.z) * (self.z - other.z))

  def is_null(self):
    return self.x == 0 and self.y == 0 and self.z == 0

  def length(self):
    return math.sqrt(self.x * self.x + self.y * self.y + self.z * self.z)
  def get_xy(self):
    return Vector(self.x, self.y)
  '''
  def scalar_add(self, scalar, scalar_y = None, scalar_z = None):
    if not scalar_y:
      return Vector3(self.x + scalar,
        self.y + scalar)
    return Vector3(self.x + scalar,
      self.y + scalar_y)
  '''
  def normalise(self):
    length = self.length()
    if length == 0:
      return
    self.x /= length
    self.y /= length
    self.z /= length

  def scale(self, x, y, z):
    return Vector3(
      self.x / x,
      self.y / y,
      self.z / z
    )
  '''  
  def scalar_sub(self, scalar_x, scalar_y):
    return Vector3(self.x - scalar_x,
      self.y - scalar.y
    )
'''
  def rotate (self,degrees):
    '''anti clockwise'''
    rad = math.pi * degrees/180
    return Vector3(
        math.cos(rad)*self.x - math.sin(rad)*self.y,
        math.sin(rad)*self.x + math.cos(rad)*self.y,
                self.z
        )
    
  def inside_quad(self, center, width, height):
    return self.x > center.x - width/2 and self.x < center.x + width/2 and self.y > center.y - height/2 and self.y < center.y + height/2

  def __sub__(self, other):
    return Vector3(
      self.x - other.x,
      self.y - other.y,
      self.z - other.z
    )
    
  def __add__(self, other):
    return Vector3(
      self.x + other.x,
      self.y + other.y,
      self.z + other.z
    )
  def __iadd__(self, other):
    self.x += other.x
    self.y += other.y
    self.z += other.z
    return self

  def __mul__(self, scalar):
    return Vector3(
      self.x * scalar,
      self.y * scalar, 
      self.z * scalar 
    )
    
  def __div__(self, scalar):
    return Vector3(
      self.x / scalar,
      self.y / scalar, 
      self.z / scalar 
    )

  def __eq__(self, other):
    return self.x == other.x and self.y == other.y and self.z == other.z

  def angle_from_x_axis(self):
    if self.x == 0 and self.y == 0:
      return 0
    elif self.x > 0 and self.y < 0:
      return 360 - math.degrees(math.acos(self.x / self.length()))
    elif self.x <= 0 and self.y < 0:
      return 180 + math.degrees(math.acos(-self.x / self.length()))
    elif self.x <= 0 and self.y >= 0:
      return 180 - math.degrees(math.acos(-self.x / self.length()))
    else:
      return math.degrees(math.acos(self.x / self.length()))
      
  def __str__(self):
    return "Vector3(%f, %f, %f)" % (self.x, self.y, self.z)

